@@ -50,7 +50,6 @@ both these contexts.
 package XML::XPathScript::Processor;
 
 use strict;
-use warnings;
 
 use Exporter;
 use vars '@ISA', '@EXPORT';
@@ -78,8 +77,6 @@ use vars '@ISA', '@EXPORT';
 		DO_TEXT_AS_CHILD
         );
 
-my $VERSION = 0.1;
-
 =pod "
 
 =over 4
@@ -148,13 +145,7 @@ values considered harmful>).
 =cut "
 
 sub findnodes {
-	if ($XML::XPathScript::xp->isa("XML::XPath")) {
-		return $XML::XPathScript::xp->findnodes(@_);
-	}
-
-	my ($path, $context) = @_;
-	$context = $XML::XPathScript::xp if (!defined $context);
-	return $context->findnodes($path);
+	$XML::XPathScript::xp->findnodes(@_)
 }
 
 =pod "
@@ -171,13 +162,7 @@ L</xpath_to_string>.
 =cut "
 
 sub findvalue {
-	if ($XML::XPathScript::xp->isa("XML::XPath")) {
-		return xpath_to_string(scalar $XML::XPathScript::xp->findvalue(@_));
-	}
-
-	my ($path, $context) = @_;
-	$context = $XML::XPathScript::xp if (!defined $context);
-	return xpath_to_string($context->findvalue($path));
+	return xpath_to_string(scalar $XML::XPathScript::xp->findvalue(@_));
 }
 
 =pod "
@@ -352,8 +337,7 @@ sub call_template {
 }
 
 sub _apply_templates {
-	no warnings 'uninitialized';
-	return join '', map translate_node($_), @_;
+	return join( '', map { translate_node($_) or "" } @_ );
 }
 
 =item  is_element_node ( $object )
@@ -605,7 +589,6 @@ sub translate_element_node {
 		unless( $trans = $translations->{$node_name} )
 		{
 			# no specific and no generic? Okay, okay, return as is...
-			no warnings qw/ uninitialized /;
 			return start_tag($node) . 
                 	_apply_templates( ( $XML::XPathScript::XML_parser eq 'XML::LibXML' ) ? 
 										$node->childNodes : $node->getChildNodes) .
@@ -673,7 +656,7 @@ sub translate_element_node {
 				if is_element_node( $kid );
         }
         
-		return ($pre||'') . ( $middle||'' ) . ($post||'');
+		return ($pre||'') . ($middle||'') . ($post||'');
     }
 	
     if($search) 
@@ -769,7 +752,7 @@ sub end_tag {
 
 sub interpolate {
     my ($node, $string) = @_;
-	
+
 	# if string is empty or no interpolation,
 	# we return
     return( $string || '' ) unless 
